/// SPDX-License-Identifier: GPL-3.0-or-later
/// SPDX-FileCopyrightText: Copyright © 2016-2025 Tony Garnock-Jones <tonyg@leastfixedpoint.com>
export class SpanResult {
    constructor(searchTarget, start) {
        this.items = [];
        this.searchTarget = searchTarget;
        this.start = start;
    }
    get offset() {
        return this.searchTarget - this.start;
    }
    get firstItem() {
        return this.items[0].item;
    }
    get lastItem() {
        return this.items[this.items.length - 1].item;
    }
}
export class SpanIndex {
    constructor() {
        this.index = [];
    }
    get(pos) {
        var _a;
        if (this.index.length === 0)
            return null;
        let lo = 0;
        let hi = this.index.length;
        // console.log(`\nsearching for ${pos}`);
        while (true) {
            if (lo === hi) {
                if (lo === 0)
                    return null;
                const e = this.index[lo - 1];
                if (e[0] > pos)
                    throw new Error("INTERNAL ERROR: bad binary search (1)");
                if (((_a = this.index[lo]) === null || _a === void 0 ? void 0 : _a[0]) <= pos)
                    throw new Error("INTERNAL ERROR: bad binary search (2)");
                // console.log(`found ${JSON.stringify(e)}, ${JSON.stringify(this.index[lo] ?? null)}`);
                const r = new SpanResult(pos, e[0]);
                e[1].forEach(([end, item]) => {
                    if (pos <= end) {
                        r.items.push({ end, item });
                    }
                });
                return (r.items.length > 0) ? r : null;
            }
            const mid = (lo + hi) >> 1;
            const e = this.index[mid];
            // console.log(`${pos} lo ${lo} hi ${hi} mid ${mid} probe ${JSON.stringify(e)}`);
            if (e[0] <= pos) {
                lo = mid + 1;
            }
            else {
                hi = mid;
            }
        }
    }
}
export class SpanInfo {
    constructor() {
        this.spans = new Map();
    }
    add(start, end, t) {
        if (!this.spans.has(start)) {
            this.spans.set(start, []);
        }
        this.spans.get(start).push([end, t]);
    }
    index() {
        const i = new SpanIndex();
        this.spans.forEach((ends, start) => {
            ends.sort((a, b) => a[0] - b[0]);
            i.index.push([start, ends]);
        });
        i.index.sort((a, b) => a[0] - b[0]);
        return i;
    }
}
//# sourceMappingURL=span.js.map