/// SPDX-License-Identifier: GPL-3.0-or-later
/// SPDX-FileCopyrightText: Copyright © 2016-2025 Tony Garnock-Jones <tonyg@leastfixedpoint.com>
import { startPos } from './position.js';
import { laxRead } from './reader.js';
import * as M from './matcher.js';
const substPat = M.scope((o) => M.seq(M.atom('$'), M.seq(M.bind(o, 'pos', M.pos), M.group('{', M.end, { skipSpace: false }))));
function toItems(readOptions, s, pos) {
    return typeof s === 'string' ? laxRead(s, Object.assign(Object.assign({}, readOptions), { start: pos, synthetic: true })) : s;
}
export class Templates {
    constructor(defaultPos = startPos(null), readOptions = {}) {
        this.sources = {};
        this.recordSources = false;
        this.defaultPos = defaultPos;
        this.readOptions = readOptions;
    }
    template(start0 = this.defaultPos, context = null) {
        const start = (typeof start0 === 'string') ? startPos(start0) : start0;
        return (consts, ...vars) => {
            var _a;
            const sourcePieces = [consts[0]];
            for (let i = 1; i < consts.length; i++) {
                sourcePieces.push('${}');
                sourcePieces.push(consts[i]);
            }
            const source = sourcePieces.join('');
            if (this.recordSources) {
                if (start.name !== null) {
                    if (start.name in this.sources && this.sources[start.name] !== source) {
                        throw new Error(`Duplicate template name: ${start.name}`);
                    }
                    this.sources[start.name] = source;
                }
            }
            let i = 0;
            return M.replace(laxRead(source, Object.assign(Object.assign({}, this.readOptions), { start, extraDelimiters: ((_a = this.readOptions.extraDelimiters) !== null && _a !== void 0 ? _a : '') + '$', synthetic: true })), context, substPat, sub => toItems(this.readOptions, vars[i++], sub.pos));
        };
    }
    sourceFor(name) {
        return this.sources[name];
    }
}
export function joinItems(itemss, separator0 = '', readOptions = {}) {
    if (itemss.length === 0)
        return [];
    const separator = toItems(readOptions, separator0, startPos(null));
    const acc = [...itemss[0]];
    for (let i = 1; i < itemss.length; i++) {
        acc.push(...separator, ...itemss[i]);
    }
    return acc;
}
export function commaJoin(itemss) {
    return joinItems(itemss, ', ');
}
export const anonymousTemplate = (new Templates()).template();
// const lib = new Templates();
// const t = (o: {xs: Items}) => lib.template('testTemplate')`YOYOYOYO ${o.xs}><`;
// console.log(t({xs: lib.template()`hello there`}));
//# sourceMappingURL=template.js.map