/// SPDX-License-Identifier: GPL-3.0-or-later
/// SPDX-FileCopyrightText: Copyright © 2016-2025 Tony Garnock-Jones <tonyg@leastfixedpoint.com>
export var TokenType;
(function (TokenType) {
    TokenType[TokenType["SPACE"] = 0] = "SPACE";
    TokenType[TokenType["NEWLINE"] = 1] = "NEWLINE";
    TokenType[TokenType["ATOM"] = 2] = "ATOM";
    TokenType[TokenType["STRING"] = 3] = "STRING";
    TokenType[TokenType["OPEN"] = 4] = "OPEN";
    TokenType[TokenType["CLOSE"] = 5] = "CLOSE";
})(TokenType || (TokenType = {}));
export function getRange(t) {
    if (Array.isArray(t)) {
        return { start: t[0].start, end: t[t.length - 1].end, synthetic: true };
    }
    else {
        return t;
    }
}
export function finishGroup(g, end) {
    return Object.assign(Object.assign({}, g), { end });
}
export function makeGroup(open, items, close) {
    return { start: open.start, open, end: close.end, close, items };
}
export function isSpace(i) {
    return isTokenType(i, TokenType.SPACE) || isTokenType(i, TokenType.NEWLINE);
}
export function isGroup(i) {
    return i && ('items' in i);
}
export function isToken(i) {
    return i && ('type' in i);
}
export function isTokenType(i, t) {
    return isToken(i) && i.type === t;
}
export function foldItems(i, fToken, fGroup, fItems) {
    const walk = (i) => {
        if (isGroup(i)) {
            return fGroup(i, fItems(i.items.map(walk)), walk);
        }
        else {
            return fToken(i);
        }
    };
    return fItems(i.map(walk));
}
export function itemText(items, options = {}) {
    return foldItems(items, i => {
        var _a;
        if ((_a = options.color) !== null && _a !== void 0 ? _a : false) {
            switch (i.type) {
                case TokenType.SPACE:
                case TokenType.NEWLINE:
                    return '\x1b[31m' + i.text + '\x1b[0m';
                case TokenType.STRING:
                    return '\x1b[34m' + i.text + '\x1b[0m';
                default:
                    return i.text;
            }
        }
        else {
            return i.text;
        }
    }, (g, inner, k) => { var _a; return k(g.open) + inner + (g.close ? k(g.close) : (_a = options.missing) !== null && _a !== void 0 ? _a : ''); }, strs => strs.join(''));
}
//# sourceMappingURL=tokens.js.map